/*
 * Copyright (c) 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)scrnctlr.h	4.1 (Berkeley) 12/4/88
 */

/*
 * definitions that have to do with the interface between the
 * controller and the screen.
 */

#define	DISP_AMPERSAND		0x30
#define	DISP_BLANK		0x10
#define	DISP_CENTSIGN		0x1b
#define	DISP_DUP		0x9f
#define	DISP_FM			0x9e
#define	DISP_GREATER_THAN	0x08
#define	DISP_NULL		0x00
#define	DISP_QUESTION		0x18
